@@ -216,10 +216,9 @@ int rsi_read_pkt(struct rsi_common *common, u8 *rx_pkt, s32 rcv_pkt_len)
 			bt_pkt_type = frame_desc[offset + BT_RX_PKT_TYPE_OFST];
 			if (bt_pkt_type == BT_CARD_READY_IND) {
 				rsi_dbg(INFO_ZONE, "BT Card ready recvd\n");
-				if (common->fsm_state == FSM_MAC_INIT_DONE)
-					rsi_attach_bt(common);
-				else
-					common->bt_defer_attach = true;
+				if (rsi_bt_ops.attach(common, &g_proto_ops))
+					rsi_dbg(ERR_ZONE,
+						"Failed to attach BT module\n");
 			} else {
 				if (common->bt_adapter)
 					rsi_bt_ops.recv_pkt(common->bt_adapter,
@@ -284,15 +283,6 @@ void rsi_set_bt_context(void *priv, void *bt_context)
 }
 #endif
 
-void rsi_attach_bt(struct rsi_common *common)
-{
-#ifdef CPTCFG_RSI_COEX
-	if (rsi_bt_ops.attach(common, &g_proto_ops))
-		rsi_dbg(ERR_ZONE,
-			"Failed to attach BT module\n");
-#endif
-}
-
 /**
  * rsi_91x_init() - This function initializes os interface operations.
  * @void: Void.
@@ -373,7 +363,6 @@ struct rsi_hw *rsi_91x_init(u16 oper_mode)
 	if (common->coex_mode > 1) {
 		if (rsi_coex_attach(common)) {
 			rsi_dbg(ERR_ZONE, "Failed to init coex module\n");
-			rsi_kill_thread(&common->tx_thread);
 			goto err;
 		}
 	}
